@@ -18,8 +18,6 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 # ***** END LICENSE BLOCK *****
 
-import abc
-
 
 ###############################################################################
 #
@@ -34,132 +32,97 @@ import abc
 class EventObserver:
     ''' adapt to event driven ECMA-35/48 parser model '''
 
-    __metaclass__ = abc.ABCMeta
-
-    @abc.abstractmethod
     def handle_start(self, context):
-        pass
+        raise NotImplementedError("EventObserver::handle_start")
 
-    @abc.abstractmethod
     def handle_end(self, context):
-        pass
+        raise NotImplementedError("EventObserver::handle_end")
 
-    @abc.abstractmethod
     def handle_csi(self, context, params, intermediate, final):
-        pass
+        raise NotImplementedError("EventObserver::handle_csi")
 
-    @abc.abstractmethod
     def handle_esc(self, context, prefix, final):
-        pass
+        raise NotImplementedError("EventObserver::handle_esc")
 
-    @abc.abstractmethod
     def handle_control_string(self, context, prefix, value):
-        pass
+        raise NotImplementedError("EventObserver::handle_control_string")
 
-    @abc.abstractmethod
     def handle_char(self, context, c):
-        pass
+        raise NotImplementedError("EventObserver::handle_char")
 
-    @abc.abstractmethod
     def handle_draw(self, context):
-        pass
+        raise NotImplementedError("EventObserver::handle_draw")
 
-    #@abc.abstractmethod
     #def handle_invalid(self, context, seq):
     #    return False
 
-    @abc.abstractmethod
     def handle_resize(self, context, row, col):
-        pass
+        raise NotImplementedError("EventObserver::handle_resize")
 
 
 class Scanner:
     ''' forward input iterator '''
 
-    __metaclass__ = abc.ABCMeta
-
-    @abc.abstractmethod
     def __iter__(self):
-        yield
+        raise NotImplementedError("Scanner::__iter__")
 
-    @abc.abstractmethod
     def assign(self, value, termenc):
-        yield
+        raise NotImplementedError("Scanner::assign")
 
 
 class OutputStream:
     ''' abstruct TTY output stream '''
 
-    __metaclass__ = abc.ABCMeta
-
-    @abc.abstractmethod
     def write(self, c):
-        pass
+        raise NotImplementedError("OutputStream::write")
 
-    @abc.abstractmethod
     def flush(self):
-        pass
+        raise NotImplementedError("OutputStream::flush")
 
 
 class EventDispatcher:
     ''' Dispatch interface of terminal sequence event oriented parser '''
 
-    __metaclass__ = abc.ABCMeta
-
-    @abc.abstractmethod
     def dispatch_esc(self, prefix, final):
-        pass
+        raise NotImplementedError("EventDispatcher::dispatch_esc")
 
-    @abc.abstractmethod
     def dispatch_csi(self, prefix, params, final):
-        pass
+        raise NotImplementedError("EventDispatcher::dispatch_csi")
 
-    @abc.abstractmethod
     def dispatch_control_string(self, prefix, value):
-        pass
+        raise NotImplementedError("EventDispatcher::dispatch_control_string")
 
-    @abc.abstractmethod
     def dispatch_char(self, c):
-        pass
+        raise NotImplementedError("EventDispatcher::dispatch_char")
 
 
 class Parser:
     ''' abstruct Parser '''
 
-    __metaclass__ = abc.ABCMeta
-
-    @abc.abstractmethod
     def parse(self, context):
-        pass
+        raise NotImplementedError("Parser::parse")
 
 
 class PTY:
     ''' abstruct PTY device '''
 
-    @abc.abstractmethod
     def fitsize(self):
-        pass
+        raise NotImplementedError("PTY::fitsize")
 
-    @abc.abstractmethod
     def resize(self, height, width):
-        pass
+        raise NotImplementedError("PTY::resize")
 
-    @abc.abstractmethod
     def read(self):
-        pass
+        raise NotImplementedError("PTY::read")
 
-    @abc.abstractmethod
     def write(self, data):
-        pass
+        raise NotImplementedError("PTY::write")
 
-    @abc.abstractmethod
     def xon(self):
-        pass
+        raise NotImplementedError("PTY::xon")
 
-    @abc.abstractmethod
     def xoff(self):
-        pass
+        raise NotImplementedError("PTY::xoff")
 
-    @abc.abstractmethod
     def drive(self):
-        pass
+        raise NotImplementedError("PTY::drive")
